@@ -13,6 +13,7 @@
 #' will also contain a sound.file column. In addition, all rows with duplicated data are removed. This is useful when 
 #' both spectrogram and waveform views are included in the Syrinx selection files. If all.data is set to \code{TRUE}) then all 
 #' columns in selection files are returned.
+#'  @seealso \code{\link{imp.raven}}
 #' @export
 #' @name imp.syrinx
 #' @examples
@@ -23,11 +24,11 @@
 #' #load data 
 #' data(selection.files)
 #' 
-#' write.table(selection.files[[3]],file = "harpyeagle.wav.txt",row.names = F,
-#'  col.names = F, sep= "\t")
+#' write.table(selection.files[[3]],file = "harpyeagle.wav.txt",row.names = FALSE,
+#'  col.names = FALSE, sep= "\t")
 #' 
-#' write.table(selection.files[[4]],file = "Phae.long4.wav.txt",row.names = F, 
-#' col.names = F, sep= "\t")
+#' write.table(selection.files[[4]],file = "Phae.long4.wav.txt",row.names = FALSE, 
+#' col.names = FALSE, sep= "\t")
 #' 
 #' syr.dat<-imp.syrinx(all.data = FALSE)
 #' 
@@ -39,16 +40,22 @@
 #' View(syr.dat)
 #' }
 #' @author Marcelo Araya-Salas (\email{araya-salas@@cornell.edu})
+#last modification on jul-5-2016 (MAS)
 
 imp.syrinx <- function(path = NULL, all.data = FALSE, recursive = FALSE) 
-{ if(is.null(path)) path = getwd()
-sel.txt <- list.files(path = path,full.names = T)
-sel.txt2 <- list.files(path = path,full.names = F)
+{ 
+  
+  #check path to working directory
+  if(!is.null(path))
+  {if(class(try(setwd(path), silent = T)) == "try-error") stop("'path' provided does not exist") else setwd(path)} #set working directory
+
+sel.txt <- list.files(full.names = TRUE)
+sel.txt2 <- list.files(full.names = FALSE)
 
 sel.txt <- sel.txt[grep(".log$|.txt$",ignore.case = TRUE, sel.txt)]
 sel.txt2 <- sel.txt2[grep(".log$|.txt$",ignore.case = TRUE, sel.txt2)]
 
-if(length(sel.txt) == 0) stop("No selection files in path provided")
+if(length(sel.txt) == 0) stop("No selection files in working directory/'path' provided")
 
 b<-NULL
 if(substring(text = readLines(sel.txt[1])[1], first = 0, last = 9) == "fieldkey:") field <- T else field <- F
@@ -56,7 +63,7 @@ if(substring(text = readLines(sel.txt[1])[1], first = 0, last = 9) == "fieldkey:
 
 clist<-lapply(1:length(sel.txt), function(i)
   {    
-  if(field)  {a <- read.table(sel.txt[i], header = T, sep = "\t", fill = TRUE) 
+  if(field)  {a <- read.table(sel.txt[i], header = TRUE, sep = "\t", fill = TRUE) 
 
   if(!all.data) { c <- data.frame(selec.file = sel.txt2[i], sound.files = a[, grep("soundfile",colnames(a))],
                                 selec = 1,
@@ -66,13 +73,13 @@ clist<-lapply(1:length(sel.txt), function(i)
   } else c<-a 
                                 b<-rbind(b, c)
                                 } else {
-          a<-read.table(sel.txt[i],header = F,sep = "\t", fill = TRUE)
+          a<-read.table(sel.txt[i],header = FALSE,sep = "\t", fill = TRUE)
             c <- a[, seq(2, ncol(a), by =2)]
-           colnames(c) <- gsub(":", "", unlist(a[1, seq(1,ncol(a), by =2)]), fixed = T)
-           if(!all.data) {c<-data.frame(sound.files = c[, grep("selected",colnames(c), ignore.case = T)],
+           colnames(c) <- gsub(":", "", unlist(a[1, seq(1,ncol(a), by =2)]), fixed = TRUE)
+           if(!all.data) {c<-data.frame(sound.files = c[, grep("selected",colnames(c), ignore.case = TRUE)],
                                        selec = 1,
-                                       start = c[, grep("lefttime",colnames(c), ignore.case = T)],
-                                       end = c[, grep("righttime",colnames(c), ignore.case = T)])
+                                       start = c[, grep("lefttime",colnames(c), ignore.case = TRUE)],
+                                       end = c[, grep("righttime",colnames(c), ignore.case = TRUE)])
            for(i in 2:nrow(c)) if(c$sound.files[i] == c$sound.files[i-1]) c$selec[i] <- c$selec[i-1] + 1}          
   return(c)
                     }
