@@ -7,14 +7,16 @@
 #'   (\url{http://www.xeno-canto.org/}).
 #' @param download Logical argument. Downloads recording file names and
 #'   associated metadata if \code{FALSE}. If \code{TRUE}, recordings are also downloaded to working
-#'   directory as .mp3 files. Default is \code{FALSE}.
+#'   directory as .mp3 files. Default is \code{FALSE}. Note that if the recording is already in the 
+#'   working directory (as when the downloading process has been interrupted) it will be skipped. 
+#'   Hence, resuming downloading processes will not start from scratch.   
 #' @param X Data frame with the same columns as the output of the function, or at least the following
 #' columns: Genus, Specific_epithet and Recording_ID. Only the recordings listed in the data frame 
 #' will be download (\code{download} argument is automatically set to \code{TRUE}). This can be used to select
 #' the recordings to be downloaded based on their attributes.  
 #' @param parallel Numeric. Controls whether parallel computing is applied.
-#' It specifies the number of cores to be used. Default is 1 (e.i. no parallel computing).
-#' For windows OS the \code{warbleR} from github to run parallel.   
+#' It specifies the number of cores to be used. Default is 1 (i.e. no parallel computing).
+#' Not available in Windows OS.
 #' @return If X is not provided the function returns a data frame with the following recording information: recording ID, Genus, Specific epithet, Subspecies, English name, Recordist, Country, Locality, Latitude, Longitude, Vocalization type, Audio file, License, URL, Quality,Time, Date. Sound files in .mp3 format are downloaded into the working directory if download = \code{TRUE} or if X is provided.
 #' @export
 #' @name querxc
@@ -24,8 +26,7 @@
 #' @examples
 #' \dontrun{
 #' # First create empty folder
-#' dir.create(file.path(getwd(),"temp"))
-#' setwd(file.path(getwd(),"temp"))
+#' setwd(tempdir())
 
 #' df1 <- querxc("Phaethornis anthophilus", download = FALSE)
 #' View(df1)
@@ -35,40 +36,35 @@
 
 #' #check this folder!!
 #' getwd()
-#' 
-#' # remove example directory
-#' unlink(getwd(),recursive = TRUE)
 #' }
-#' @author Marcelo Araya-Salas (\email{araya-salas@@cornell.edu}) and Hua Zhong
+#' @author Marcelo Araya-Salas (\email{araya-salas@@cornell.edu}) 
+#last modification on jul-5-2016 (MAS)
 
-querxc <- function(qword, download=FALSE, X = NULL, parallel = 1) {
+querxc <- function(qword, download = FALSE, X = NULL, parallel = 1) {
   # If parallel is not numeric
   if(!is.numeric(parallel)) stop("'parallel' must be a numeric vector of length 1") 
   if(any(!(parallel %% 1 == 0),parallel < 1)) stop("'parallel' should be a positive integer")
   
-  #if on windows you need parallelsugar package
-  if(parallel > 1)
-  { 
-    #      options(warn = -1)
-    #      
-    #        
-    #        if(Sys.info()[1] == "Windows"){ 
-    #       cat 
-    #       lapp <- pbapply::pblapply} else 
-    lapp <- function(X, FUN) parallel::mclapply(X, FUN, mc.cores = parallel)} else lapp <- pbapply::pblapply
+  #parallel not available on windows
+  if(parallel > 1 & Sys.info()[1] == "Windows")
+  {message("parallel computing not availabe in Windows OS for this function")
+    parallel <- 1}
+  
+  if(parallel > 1) 
+lapp <- function(X, FUN) parallel::mclapply(X, FUN, mc.cores = parallel) else lapp <- pbapply::pblapply
     
           options(warn = 0)
   
   if(is.null(X))
   {
   #check internet connection
-  a <- try(RCurl::getURL("www.xeno-canto.org"), silent=T)
+  a <- try(RCurl::getURL("www.xeno-canto.org"), silent = TRUE)
   if(substr(a[1],0,5) == "Error") stop("No connection to xeno-canto.org (check your internet connection!)")
 
   if(a == "Could not connect to the database")  stop("xeno-canto.org website is apparently down")
   
   #search recs in xeno-canto (results are returned in pages with 500 recordings each)
-  cat("Obtaining recording list...")
+  message("Obtaining recording list...")
   if(sapply(strsplit(qword, " "), length) == 2)
   query <- rjson::fromJSON(, paste("http://www.xeno-canto.org/api/recordings.php?species_nr=&query=", #run search
                             strsplit(qword, " ")[[1]][1],"%20",strsplit(qword, " ")[[1]][2], sep="")) else
@@ -91,7 +87,7 @@ querxc <- function(qword, download=FALSE, X = NULL, parallel = 1) {
     recs <- c(recs, query$recordings)
   }
   
-  cat("Processing recording information:")
+  message("Processing recording information:")
   
   results <- as.data.frame(t(sapply(matrix(c(1:n.recs), ncol=1), function(x){
 
@@ -130,7 +126,7 @@ querxc <- function(qword, download=FALSE, X = NULL, parallel = 1) {
   #remove duplicates
 results <- results[!duplicated(results$Recording_ID), ]
 
-cat(paste( nrow(results), " recordings found!", sep=""))  
+message(paste( nrow(results), " recordings found!", sep=""))  
 
 } else { 
   #stop if X is not a data frame
@@ -143,6 +139,7 @@ results <- X  }
 
   #download recordings
   if(download) {
+    message("Downloading sound files...")
     lapp(matrix(c(1:length(results$Genus)), ncol=1), function(x){
       gen <- results$Genus[x]
       se <- results$Specific_epithet[x]
@@ -155,7 +152,7 @@ results <- X  }
       return (NULL)
     })
     
-    cat("double-checking downloaded files")
+    message("double-checking downloaded files")
    
     fl <- list.files(pattern = ".mp3$")
     size0 <- fl[file.size(fl) == 0]
@@ -164,7 +161,7 @@ results <- X  }
 {   
  unlink(size0)
     
-    s0df <- data.frame(Genus = sapply(strsplit(as.character(size0), "-",fixed=T), "[",1), Specific_epithet = sapply(strsplit(as.character(size0), "-",fixed=T), "[",2), Recording_ID = gsub(".mp3", "",sapply(strsplit(as.character(size0), "-",fixed=T), "[",3)))
+    s0df <- data.frame(Genus = sapply(strsplit(as.character(size0), "-",fixed = TRUE), "[",1), Specific_epithet = sapply(strsplit(as.character(size0), "-",fixed = TRUE), "[",2), Recording_ID = gsub(".mp3", "",sapply(strsplit(as.character(size0), "-",fixed = TRUE), "[",3)))
  
     lapp(matrix(c(1:length(s0df$Genus)), ncol=1), function(x){
       gen <- s0df$Genus[x]
