@@ -1,15 +1,17 @@
 #' Measure signal-to-noise ratio
 #' 
 #' \code{sig2noise} measures signal-to-noise ratio across multiple files.
-#' @usage sig2noise(X, mar, parallel = 1)
+#' @usage sig2noise(X, mar, parallel = 1, path = NULL)
 #' @param X Data frame with results from \code{\link{manualoc}} or any data frame with columns
 #' for sound file name (sound.files), selection number (selec), and start and end time of signal
 #' (start and end). 
 #' @param mar numeric vector of length 1. Specifies the margins adjacent to
 #'   the start and end points of selection over which to measure noise.
 #' @param parallel Numeric. Controls whether parallel computing is applied.
-#' It specifies the number of cores to be used. Default is 1 (e.i. no parallel computing).
-#' For windows OS the \code{warbleR} from github to run parallel.   
+#' It specifies the number of cores to be used. Default is 1 (i.e. no parallel computing).
+#' Not available in Windows OS.
+#' @param path Character string containing the directory path where the sound files are located. 
+#' If \code{NULL} (default) then the current working directory is used.
 #' @return Data frame similar to \code{\link{autodetec}} output, but also includes a new variable 
 #' with the signal-to-noise values.
 #' @export
@@ -43,77 +45,73 @@
 #' 
 #' @author Marcelo Araya-Salas (\email{araya-salas@@cornell.edu}) and Grace Smith Vidaurre
 #' @source \url{https://en.wikipedia.org/wiki/Signal-to-noise_ratio}
+#last modification on jul-5-2016 (MAS)
 
-sig2noise <- function(X, mar, parallel = 1){
-  if(class(X) == "data.frame") {if(all(c("sound.files", "selec", 
-                                         "start", "end") %in% colnames(X))) 
-  {
-    start <- as.numeric(unlist(X$start))
-    end <- as.numeric(unlist(X$end))
-    sound.files <- as.character(unlist(X$sound.files))
-    selec <- as.character(unlist(X$selec))
-  } else stop(paste(paste(c("sound.files", "selec", "start", "end")[!(c("sound.files", "selec", 
-                                                                       "start", "end") %in% colnames(X))], collapse=", "), "column(s) not found in data frame"))
-  } else  stop("X is not a data frame")
+sig2noise <- function(X, mar, parallel = 1, path = NULL){
+  
+  #check path to working directory
+  if(!is.null(path))
+  {if(class(try(setwd(path), silent = T)) == "try-error") stop("'path' provided does not exist") else setwd(path)} #set working directory
+  
+  #if X is not a data frame
+  if(!class(X) == "data.frame") stop("X is not a data frame")
+  
+  if(!all(c("sound.files", "selec", 
+            "start", "end") %in% colnames(X))) 
+    stop(paste(paste(c("sound.files", "selec", "start", "end")[!(c("sound.files", "selec", 
+                                                                   "start", "end") %in% colnames(X))], collapse=", "), "column(s) not found in data frame"))
   
   #if there are NAs in start or end stop
-  if(any(is.na(c(end, start)))) stop("NAs found in start and/or end")  
+  if(any(is.na(c(X$end, X$start)))) stop("NAs found in start and/or end")  
   
   #if end or start are not numeric stop
-  if(all(class(end) != "numeric" & class(start) != "numeric")) stop("'end' and 'selec' must be numeric")
+  if(all(class(X$end) != "numeric" & class(X$start) != "numeric")) stop("'end' and 'selec' must be numeric")
   
   #if any start higher than end stop
-  if(any(end - start<0)) stop(paste("The start is higher than the end in", length(which(end - start<0)), "case(s)"))  
+  if(any(X$end - X$start<0)) stop(paste("The start is higher than the end in", length(which(X$end - X$start<0)), "case(s)"))  
   
   #if any selections longer than 20 secs stop
-  if(any(end - start>20)) stop(paste(length(which(end - start>20)), "selection(s) longer than 20 sec"))  
-  options( show.error.messages = TRUE)
+  if(any(X$end - X$start>20)) stop(paste(length(which(X$end - X$start>20)), "selection(s) longer than 20 sec"))  
+  
+   options( show.error.messages = TRUE)
   
   #return warning if not all sound files were found
-  fs <- list.files(path = getwd(), pattern = ".wav$", ignore.case = T)
-  if(length(unique(sound.files[(sound.files %in% fs)])) != length(unique(sound.files))) 
-    cat(paste(length(unique(sound.files))-length(unique(sound.files[(sound.files %in% fs)])), 
+  fs <- list.files(path = getwd(), pattern = ".wav$", ignore.case = TRUE)
+  if(length(unique(X$sound.files[(X$sound.files %in% fs)])) != length(unique(X$sound.files))) 
+    message(paste(length(unique(X$sound.files))-length(unique(X$sound.files[(X$sound.files %in% fs)])), 
                   ".wav file(s) not found"))
   
   #count number of sound files in working directory and if 0 stop
-  d <- which(sound.files %in% fs) 
+  d <- which(X$sound.files %in% fs) 
   if(length(d) == 0){
     stop("The .wav files are not in the working directory")
-  }  else {
-    start <- start[d]
-    end <- end[d]
-    selec <- selec[d]
-    sound.files <- sound.files[d]
-  }
+  }  else X <- X[d, ]
    
   # If parallel is not numeric
   if(!is.numeric(parallel)) stop("'parallel' must be a numeric vector of length 1") 
   if(any(!(parallel %% 1 == 0),parallel < 1)) stop("'parallel' should be a positive integer")
   
-  #if on windows you need parallelsugar package
-  if(parallel > 1)
-  { 
-    #      options(warn = -1)
-    #      
-    #        
-    #        if(Sys.info()[1] == "Windows"){ 
-    #       cat 
-    #       lapp <- pbapply::pblapply} else 
-    lapp <- function(X, FUN) parallel::mclapply(X, FUN, mc.cores = parallel)} else lapp <- pbapply::pblapply
+  #parallel not available on windows
+  if(parallel > 1 & Sys.info()[1] == "Windows")
+  {message("parallel computing not availabe in Windows OS for this function")
+    parallel <- 1}
+  
+  if(parallel > 1) 
+    lapp <- function(X, FUN) parallel::mclapply(X, FUN, mc.cores = parallel) else lapp <- pbapply::pblapply
   
   options(warn = 0)
   
-  SNR <- lapp(c(1:length(sound.files)), function(y){
+  SNR <- lapp(c(1:length(X$sound.files)), function(y){
       
     # Read sound files to get sample rate and length
-      r <- tuneR::readWave(file.path(getwd(), sound.files[y]), header = TRUE)
+      r <- tuneR::readWave(file.path(getwd(), X$sound.files[y]), header = TRUE)
       f <- r$sample.rate
     
       #reset coordinates of signals 
-      stn <- start[y] - mar
-      enn <- end[y] + mar
+      stn <- X$start[y] - mar
+      enn <- X$end[y] + mar
       mar1 <- mar
-      mar2 <- mar1 + end[y] - start[y]
+      mar2 <- mar1 + X$end[y] - X$start[y]
       
       if (stn < 0) { 
       mar1 <- mar1  + stn
@@ -123,7 +121,7 @@ sig2noise <- function(X, mar, parallel = 1){
       
       if(enn > r$samples/f) enn <- r$samples/f
       
-      r <- tuneR::readWave(file.path(getwd(), sound.files[y]), from = stn, to = enn, units = "seconds")
+      r <- tuneR::readWave(file.path(getwd(), X$sound.files[y]), from = stn, to = enn, units = "seconds")
       
       # Identify the signal
       signal <- seewave::cutw(r, from =  mar1, to = mar2, f = f)
