@@ -2,11 +2,11 @@
 #' 
 #' \code{snrspecs} creates spectrograms to visualize margins over which background noise
 #' will be measured by \code{\link{sig2noise}}.
-#' @usage snrspecs(X, wl = 512, flim = c(0, 22), wn = "hanning", pal =
-#'   reverse.gray.colors.2, ovlp = 70, inner.mar = c(5, 4, 4, 2), outer.mar =
-#'   c(0, 0, 0, 0), picsize = 1, res = 100, cexlab = 1, title = TRUE, 
-#'   propwidth= FALSE, xl=1, osci = FALSE, gr = FALSE, sc = FALSE, mar =
-#'   0.2, snrmar = 0.1, it = "jpeg", parallel = 1)
+#' @usage snrspecs(X, wl = 512, flim = c(0, 22), wn = "hanning", ovlp = 70, 
+#' inner.mar = c(5, 4, 4, 2), outer.mar = c(0, 0, 0, 0), picsize = 1, 
+#' res = 100, cexlab = 1, title = TRUE, 
+#'   propwidth= FALSE, xl=1, osci = FALSE, gr = FALSE, sc = FALSE, mar = 0.2,
+#'    snrmar = 0.1, it = "jpeg", parallel = 1, path = NULL)
 #' @param  X Data frame with results from \code{\link{manualoc}} or any data frame with columns
 #' for sound file name (sound.files), selection number (selec), and start and end time of signal
 #' (start and end). 
@@ -16,8 +16,6 @@
 #'   the spectrogram, as in \code{\link[seewave]{spectro}}. Default is c(0, 22).
 ##' @param wn Character vector of length 1 specifying window name. Default is 
 #'   "hanning". See function \code{\link[seewave]{ftwindow}} for more options.
-#' @param pal A color palette function to be used to assign colors in the 
-#'   plot, as in \code{\link[seewave]{spectro}}. Default is reverse.gray.colors.2. See Details.
 #' @param ovlp Numeric vector of length 1 specifying \% of overlap between two 
 #'   consecutive windows, as in \code{\link[seewave]{spectro}}. Default is 70.
 #' @param inner.mar Numeric vector with 4 elements, default is c(5,4,4,2). 
@@ -51,10 +49,9 @@
 #' @param it A character vector of length 1 giving the image type to be used. Currently only
 #' "tiff" and "jpeg" are admitted. Default is "jpeg".
 #' @param parallel Numeric. Controls whether parallel computing is applied.
-#' It specifies the number of cores to be used. Default is 1 (e.i. no parallel computing).
-#' For windows OS the \code{warbleR} from github to run parallel. 
-#'   Note that creating images is not compatible with parallel computing 
-#'   (parallel > 1) in OSX (mac).   
+#' It specifies the number of cores to be used. Default is 1 (i.e. no parallel computing).
+#' @param path Character string containing the directory path where the sound files are located. 
+#' If \code{NULL} (default) then the current working directory is used.
 #' @return Spectrograms per selection marked with margins where background noise will be measured.
 #' @family spectrogram creators
 #' @seealso \code{\link{trackfreqs}} for creating spectrograms to visualize 
@@ -73,11 +70,9 @@
 #' @examples
 #' \dontrun{
 #' # First create empty folder
-#' dir.create(file.path(getwd(),"temp"))
-#' setwd(file.path(getwd(),"temp"))
-#' 
-#' data(list = c("Phae.long1", "Phae.long2"))
-#' data(manualoc.df)
+#' setwd(tempdir())
+#'  
+#' data(list = c("Phae.long1", "Phae.long2", "manualoc.df"))
 #' writeWave(Phae.long1, "Phae.long1.wav") #save sound.files
 #' writeWave(Phae.long2, "Phae.long2.wav") 
 #' 
@@ -85,104 +80,86 @@
 #' # snrmar needs to be smaller before moving on to sig2noise()
 #' 
 #' snrspecs(manualoc.df, flim = c(0, 14), inner.mar = c(4,4.5,2,1), outer.mar = c(4,2,2,1), 
-#' picsize = 2, res = 300, cexlab = 2, mar = 0.2, snrmar = 0.1, it = "jpeg")
+#' picsize = 2, res = 300, cexlab = 2, mar = 0.2, snrmar = 0.1, it = "jpeg", wl = 300)
 #' 
 #' # make only Phae.long1 spectrograms
 #' # snrmar now doesn't overlap neighboring signals
 #' 
 #' snrspecs(manualoc.df[grepl(c("Phae.long1"), manualoc.df$sound.files), ], flim = c(3, 14), 
 #' inner.mar = c(4,4.5,2,1), outer.mar = c(4,2,2,1), picsize = 2, res = 300, cexlab = 2, 
-#' mar = 0.2, snrmar = 0.01)
+#' mar = 0.2, snrmar = 0.01, wl = 300)
 #' 
 #' #check this folder!!
 #' getwd()
-#' 
-#' #remove example directory
-#' unlink(getwd(),recursive = TRUE)
 #' }
 #' @author Marcelo Araya-Salas (\email{araya-salas@@cornell.edu}) and Grace Smith Vidaurre
 #' @source \url{https://en.wikipedia.org/wiki/Signal-to-noise_ratio}
+#last modification on jul-5-2016 (MAS)
 
-snrspecs <- function(X, wl = 512, flim = c(0, 22), wn = "hanning", pal = reverse.gray.colors.2, ovlp = 70,
+snrspecs <- function(X, wl = 512, flim = c(0, 22), wn = "hanning", ovlp = 70,
                      inner.mar = c(5,4,4,2), outer.mar = c(0,0,0,0), picsize = 1, res = 100,
                      cexlab = 1, title = TRUE, propwidth = FALSE, xl = 1, osci = FALSE, 
-                     gr = FALSE, sc = FALSE, mar = 0.2, snrmar = 0.1, it = "jpeg", parallel = 1){
-
-  if(class(X) == "data.frame") {if(all(c("sound.files", "selec", 
-                                         "start", "end") %in% colnames(X))){
-    start <- as.numeric(unlist(X$start))
-    end <- as.numeric(unlist(X$end))
-    sound.files <- as.character(unlist(X$sound.files))
-    selec <- as.character(unlist(X$selec))
-    
-    } else stop(paste(paste(c("sound.files", "selec", "start", "end")[!(c("sound.files", "selec", 
-                                                                       "start", "end") %in% colnames(X))], collapse=", "), "column(s) not found in data frame"))
-  } else  stop("X is not a data frame")
+                     gr = FALSE, sc = FALSE, mar = 0.2, snrmar = 0.1, it = "jpeg", parallel = 1,
+                     path = NULL){
+ 
+  #check path to working directory
+  if(!is.null(path))
+  {if(class(try(setwd(path), silent = T)) == "try-error") stop("'path' provided does not exist") else setwd(path)} #set working directory
+  
+  
+  #if X is not a data frame
+  if(!class(X) == "data.frame") stop("X is not a data frame")
+  
+  if(!all(c("sound.files", "selec", 
+            "start", "end") %in% colnames(X))) 
+    stop(paste(paste(c("sound.files", "selec", "start", "end")[!(c("sound.files", "selec", 
+                                                                   "start", "end") %in% colnames(X))], collapse=", "), "column(s) not found in data frame"))
   
   #if there are NAs in start or end stop
-  if(any(is.na(c(end, start)))) stop("NAs found in start and/or end")  
+  if(any(is.na(c(X$end, X$start)))) stop("NAs found in start and/or end")  
   
   #if end or start are not numeric stop
-  if(all(class(end) != "numeric" & class(start) != "numeric")) stop("'end' and 'selec' must be numeric")
+  if(all(class(X$end) != "numeric" & class(X$start) != "numeric")) stop("'end' and 'selec' must be numeric")
   
   #if any start higher than end stop
-  if(any(end - start<0)) stop(paste("The start is higher than the end in", length(which(end - start<0)), "case(s)"))  
+  if(any(X$end - X$start<0)) stop(paste("The start is higher than the end in", length(which(X$end - X$start<0)), "case(s)"))  
   
   #if it argument is not "jpeg" or "tiff" 
   if(!any(it == "jpeg", it == "tiff")) stop(paste("Image type", it, "not allowed"))  
   
   #if any selections longer than 20 secs stop
-  if(any(end - start>20)) stop(paste(length(which(end - start>20)), "selection(s) longer than 20 sec"))  
+  if(any(X$end - X$start>20)) stop(paste(length(which(X$end - X$start>20)), "selection(s) longer than 20 sec"))  
   options( show.error.messages = TRUE)
     
   #return warning if not all sound files were found
-  fs <- list.files(path = getwd(), pattern = ".wav$", ignore.case = TRUE)
-  if(length(unique(sound.files[(sound.files %in% fs)])) != length(unique(sound.files))) 
-    cat(paste(length(unique(sound.files))-length(unique(sound.files[(sound.files %in% fs)])), 
+  fs <- list.files(pattern = ".wav$", ignore.case = TRUE)
+  if(length(unique(X$sound.files[(X$sound.files %in% fs)])) != length(unique(X$sound.files))) 
+    message(paste(length(unique(X$sound.files))-length(unique(X$sound.files[(X$sound.files %in% fs)])), 
                   ".wav file(s) not found"))
   
   #count number of sound files in working directory and if 0 stop
-  d <- which(sound.files %in% fs) 
+  d <- which(X$sound.files %in% fs) 
   if(length(d) == 0){
     stop("The .wav files are not in the working directory")
-  }  else {
-    start <- start[d]
-    end <- end[d]
-    selec <- selec[d]
-    sound.files <- sound.files[d]
-  }
+  }  else X <- X[d, ]
   
   # If parallel is not numeric
   if(!is.numeric(parallel)) stop("'parallel' must be a numeric vector of length 1") 
   if(any(!(parallel %% 1 == 0),parallel < 1)) stop("'parallel' should be a positive integer")
   
   #if parallel in OSX
-  if(all(parallel > 1, !Sys.info()[1] %in% c("Linux","Windows"))) {
-    parallel <- 1
-    cat("creating images is not compatible with parallel computing (parallel > 1) in OSX (mac)")
-  }
+  # if(all(parallel > 1, !Sys.info()[1] %in% c("Linux","Windows"))) {
+  #   parallel <- 1
+  #   message("creating images is not compatible with parallel computing (parallel > 1) in OSX (mac)")
+  # }
   
-  #if on windows you need parallelsugar package
-  if(parallel > 1)
-  { 
-    #      options(warn = -1)
-    #      
-    #        
-    #        if(Sys.info()[1] == "Windows"){ 
-    #       cat 
-    #       lapp <- pbapply::pblapply} else 
-    lapp <- function(X, FUN) parallel::mclapply(X, FUN, mc.cores = parallel)} else lapp <- pbapply::pblapply
-  
-  options(warn = 0)
-  
-  if(parallel == 1) cat("Creating spectrograms with signal and noise margins to be used in sig2noise():")
-  invisible(lapp(1:length(sound.files), function(i){
+    snrspeFUN <- function(i, X, wl, flim, ovlp, inner.mar, outer.mar, picsize, res, cexlab, xl, mar, snrmar){
     
-    r <- tuneR::readWave(file.path(getwd(), sound.files[i])) 
+    r <- tuneR::readWave(as.character(X$sound.files[i])) 
     f <- r@samp.rate
     
     # Read sound files to get sample rate and length
-    r <- tuneR::readWave(file.path(getwd(), sound.files[i]), header = TRUE)
+    r <- tuneR::readWave(as.character(X$sound.files[i]), header = TRUE)
     f <- r$sample.rate
     
     fl<- flim #in case flim its higher than can be due to sampling rate
@@ -193,10 +170,10 @@ snrspecs <- function(X, wl = 512, flim = c(0, 22), wn = "hanning", pal = reverse
     if(mar < snrmar) mar <- snrmar
     
     # Correct start and end time if is smaller than 0 or higher than length of rec
-    st <- start[i] - mar
-    en <- end[i] + mar
+    st <- X$start[i] - mar
+    en <- X$end[i] + mar
     mar1 <- mar
-    mar2 <- mar1 + end[i] - start[i]
+    mar2 <- mar1 + X$end[i] - X$start[i]
     
     if (st < 0)  {
       mar1 <- mar1  + st
@@ -206,20 +183,20 @@ snrspecs <- function(X, wl = 512, flim = c(0, 22), wn = "hanning", pal = reverse
   
     if(en > r$samples/f) en <- r$samples/f
 
-    r <- tuneR::readWave(file.path(getwd(), sound.files[i]), from = st, to = en, units = "seconds")
+    r <- tuneR::readWave(as.character(X$sound.files[i]), from = st, to = en, units = "seconds")
     
     
 # Spectrogram width can be proportional to signal duration
     if(propwidth == TRUE){
-      if(it == "tiff")  tiff(filename = paste(sound.files[i],"-", selec[i], "-", "snr", ".tiff", sep = ""), 
+      if(it == "tiff")  tiff(filename = paste(X$sound.files[i],"-", X$selec[i], "-", "snr", ".tiff", sep = ""), 
            width = (10.16) * ((t[2]-t[1])/0.27) * xl * picsize, height = (10.16) * picsize, units = "cm", res = res) else
-             jpeg(filename = paste(sound.files[i],"-", selec[i], "-", "snr", ".jpeg", sep = ""), 
+             jpeg(filename = paste(X$sound.files[i],"-", X$selec[i], "-", "snr", ".jpeg", sep = ""), 
                   width = (10.16) * ((t[2]-t[1])/0.27) * xl * picsize, height = (10.16) * picsize, units = "cm", res = res)
       
     } else {
-      if(it == "tiff")  tiff(filename = paste(sound.files[i],"-", selec[i], "-", "snr", ".tiff", sep = ""), 
+      if(it == "tiff")  tiff(filename = paste(X$sound.files[i],"-", X$selec[i], "-", "snr", ".tiff", sep = ""), 
            width = (10.16) * xl * picsize, height = (10.16) * picsize, units = "cm", res = res) else
-             jpeg(filename = paste(sound.files[i],"-", selec[i], "-", "snr", ".jpeg", sep = ""), 
+             jpeg(filename = paste(X$sound.files[i],"-", X$selec[i], "-", "snr", ".jpeg", sep = ""), 
                   width = (10.16) * xl * picsize, height = (10.16) * picsize, units = "cm", res = res)
     }
     
@@ -238,12 +215,12 @@ snrspecs <- function(X, wl = 512, flim = c(0, 22), wn = "hanning", pal = reverse
 
     # Generate spectrogram using seewave
     seewave::spectro(r, f = f, wl = wl, ovlp = ovlp, collevels = seq(-40, 0, 0.5), heights = hts,
-            wn = "hanning", widths = wts, palette = pal, osc = osci, grid = gr, scale = sc, 
+            wn = "hanning", widths = wts, palette = seewave::reverse.gray.colors.2, osc = osci, grid = gr, scale = sc, 
             collab = "black", cexlab = cexlab, cex.axis = 0.5*picsize, tlab = "Time (s)", 
             flab = "Frequency (kHz)", flim = fl, alab = "", trel = FALSE)
     
     if(title){
-      title(paste(sound.files[i], "-", selec[i], "-", "snr", sep = ""), cex.main = cexlab)
+      title(paste(X$sound.files[i], "-", X$selec[i], "-", "snr", sep = ""), cex.main = cexlab)
     }
     
     # Add lines to visualize signal
@@ -265,7 +242,30 @@ snrspecs <- function(X, wl = 512, flim = c(0, 22), wn = "hanning", pal = reverse
     
     dev.off()  
     return (NULL)
-    
-  })) 
-}
+  }
+      # Run parallel in windows
+      if(parallel > 1) {if(Sys.info()[1] == "Windows") {
+        
+        cl <- parallel::makeCluster(parallel)
+        
+        doParallel::registerDoParallel(cl)
+        
+        a1 <- parallel::parLapply(cl, 1:nrow(X), function(i)
+        {
+          snrspeFUN(X = X, i = i, wl = wl, flim = flim, ovlp = ovlp, inner.mar = inner.mar, outer.mar = outer.mar, picsize = picsize, res = res, cexlab = cexlab, xl = xl, mar = mar, snrmar = snrmar)
+        })
+        
+        parallel::stopCluster(cl)
+        
+        
+      } else {    # Run parallel in other operating systems
+        
+        a1 <- parallel::mclapply(1:nrow(X), function (i) {
+          snrspeFUN(X = X, i = i, wl = wl, flim = flim, ovlp = ovlp, inner.mar = inner.mar, outer.mar = outer.mar, picsize = picsize, res = res, cexlab = cexlab, xl = xl, mar = mar, snrmar = snrmar)
+        })
+      }
+      }
+      else {sp <- pbapply::pblapply(1:nrow(X), function(i) snrspeFUN(X = X, i = i, wl = wl, flim = flim, ovlp = ovlp, inner.mar = inner.mar, outer.mar = outer.mar, picsize = picsize, res = res, cexlab = cexlab, xl = xl, mar = mar, snrmar = snrmar))
+      }
+    }
 
