@@ -5,7 +5,7 @@
 #'   = reverse.gray.colors.2, ovlp = 70, inner.mar = c(5, 4, 4, 2), outer.mar =
 #'   c(0, 0, 0, 0), picsize = 1, res = 100, cexlab = 1, title = TRUE,
 #'   propwidth = FALSE, xl = 1, osci = FALSE, gr = FALSE,  sc = FALSE, line = TRUE,
-#'   mar = 0.05, it = "jpeg", parallel = 1)
+#'   mar = 0.05, it = "jpeg", parallel = 1, path = NULL)
 #' @param  X Data frame with results containing columns for sound file name (sound.files), 
 #' selection number (selec), and start and end time of signals (start and end).
 #' The ouptut of \code{\link{manualoc}} or \code{\link{autodetec}} can be used as the input data frame. 
@@ -49,10 +49,9 @@
 #' @param it A character vector of length 1 giving the image type to be used. Currently only
 #' "tiff" and "jpeg" are admitted. Default is "jpeg".
 #' @param parallel Numeric. Controls whether parallel computing is applied.
-#' It specifies the number of cores to be used. Default is 1 (e.i. no parallel computing).
-#' For windows OS the \code{warbleR} from github to run parallel. 
-#'   Note that creating images is not compatible with parallel computing 
-#'   (parallel > 1) in OSX (mac).   
+#' It specifies the number of cores to be used. Default is 1 (i.e. no parallel computing).
+#' @param path Character string containing the directory path where the sound files are located. 
+#' If \code{NULL} (default) then the current working directory is used.
 #' @return Image files containing spectrograms of the signals listed in the input data frame.
 #' @family spectrogram creators
 #' @seealso \code{\link{trackfreqs}} for creating spectrograms to visualize 
@@ -60,7 +59,7 @@
 #'   creating spectrograms to optimize noise margins used in \code{\link{sig2noise}}
 #' @export
 #' @name specreator
-#' @details This function creates spectrograms for visualization of vocalizations. 
+#' @details This function provides access to bath process of the \code{\link[seewave]{spectro}} function from the 'seewave' package. The function creates spectrograms for visualization of vocalizations. 
 #' Setting inner.mar to c(4,4.5,2,1) and outer.mar to c(4,2,2,1) works well when picsize = 2 or 3. 
 #' Title font size, inner.mar and outer.mar (from mar and oma) don't work well when osci or sc = TRUE,
 #' this may take some optimization by the user.
@@ -76,64 +75,60 @@
 #' # make spectrograms
 #' 
 #' specreator(manualoc.df, flim = c(0, 11), inner.mar = c(4,4.5,2,1), outer.mar = c(4,2,2,1), 
-#'           picsize = 2, res = 300, cexlab = 2, mar = 0.05)
+#'           picsize = 2, res = 300, cexlab = 2, mar = 0.05, wl = 300)
 
 #' #check this folder!!
 #' getwd()
 #' }
 #' 
 #' @author Marcelo Araya-Salas (\email{araya-salas@@cornell.edu}) and Grace Smith Vidaurre
+#last modification on jul-5-2016 (MAS)
 
 specreator <- function(X, wl = 512, flim = c(0, 22), wn = "hanning", pal = reverse.gray.colors.2, ovlp = 70, 
                         inner.mar = c(5,4,4,2), outer.mar = c(0,0,0,0), picsize = 1, res = 100, 
                         cexlab = 1, title = TRUE, propwidth = FALSE, xl=1, osci = FALSE, 
-                        gr = FALSE, sc = FALSE, line = TRUE, mar = 0.05, it = "jpeg", parallel = 1){
+                        gr = FALSE, sc = FALSE, line = TRUE, mar = 0.05, it = "jpeg", parallel = 1, path = NULL){
   
+  #check path to working directory
+  if(!is.null(path))
+  {if(class(try(setwd(path), silent = T)) == "try-error") stop("'path' provided does not exist") else setwd(path)} #set working directory
   
-  if(class(X) == "data.frame") {if(all(c("sound.files", "selec", 
-                                         "start", "end") %in% colnames(X))) 
-  {
-    start <- as.numeric(unlist(X$start))
-    end <- as.numeric(unlist(X$end))
-    sound.files <- as.character(unlist(X$sound.files))
-    selec <- as.character(unlist(X$selec))
-    selcom <- as.character(unlist(X$sel.comment))
-    
-  } else stop(paste(paste(c("sound.files", "selec", "start", "end", "sel.comment")[!(c("sound.files", "selec", 
-                                                                                       "start", "end", "sel.comment") %in% colnames(X))], collapse=", "), "column(s) not found in data frame"))
-  } else stop("X is not a data frame")
+  #if X is not a data frame
+  if(!class(X) == "data.frame") stop("X is not a data frame")
+  
+  if(!all(c("sound.files", "selec", 
+            "start", "end") %in% colnames(X))) 
+    stop(paste(paste(c("sound.files", "selec", "start", "end")[!(c("sound.files", "selec", 
+                                                                   "start", "end") %in% colnames(X))], collapse=", "), "column(s) not found in data frame"))
   
   #if there are NAs in start or end stop
-  if(any(is.na(c(end, start)))) stop("NAs found in start and/or end")  
+  if(any(is.na(c(X$end, X$start)))) stop("NAs found in start and/or end")  
   
   #if end or start are not numeric stop
-  if(all(class(end) != "numeric" & class(start) != "numeric")) stop("'end' and 'selec' must be numeric")
+  if(all(class(X$end) != "numeric" & class(X$start) != "numeric")) stop("'end' and 'selec' must be numeric")
   
   #if any start higher than end stop
-  if(any(end - start<0)) stop(paste("The start is higher than the end in", length(which(end - start<0)), "case(s)"))  
+  if(any(X$end - X$start<0)) stop(paste("The start is higher than the end in", length(which(X$end - X$start<0)), "case(s)"))  
   
   #if any selections longer than 20 secs stop
-  if(any(end - start>20)) stop(paste(length(which(end - start>20)), "selection(s) longer than 20 sec"))  
+  if(any(X$end - X$start>20)) stop(paste(length(which(X$end - X$start>20)), "selection(s) longer than 20 sec"))  
   options( show.error.messages = TRUE)
   
   #if it argument is not "jpeg" or "tiff" 
   if(!any(it == "jpeg", it == "tiff")) stop(paste("Image type", it, "not allowed"))  
   
   #return warning if not all sound files were found
-  recs.wd <- list.files(path = getwd(), pattern = ".wav$", ignore.case = T)
-  if(length(unique(sound.files[(sound.files %in% recs.wd)])) != length(unique(sound.files))) 
-    (paste(length(unique(sound.files))-length(unique(sound.files[(sound.files %in% recs.wd)])), 
-                  ".wav file(s) not found"))
+  recs.wd <- list.files(pattern = ".wav$", ignore.case = TRUE)
+  if(length(unique(X$sound.files[(X$sound.files %in% recs.wd)])) != length(unique(X$sound.files))) 
+    (paste(length(unique(X$sound.files))-length(unique(X$sound.files[(X$sound.files %in% recs.wd)])), 
+           ".wav file(s) not found"))
   
   #count number of sound files in working directory and if 0 stop
-  d <- which(sound.files %in% recs.wd) 
+  d <- which(X$sound.files %in% recs.wd) 
   if(length(d) == 0){
     stop("The .wav files are not in the working directory")
   }  else {
-    start <- start[d]
-    end <- end[d]
-    selec <- selec[d]
-    sound.files <- sound.files[d]
+    X <- X[d, ]
   }
   
   if(propwidth) picsize <- 1
@@ -142,36 +137,16 @@ specreator <- function(X, wl = 512, flim = c(0, 22), wn = "hanning", pal = rever
   if(!is.numeric(parallel)) stop("'parallel' must be a numeric vector of length 1") 
   if(any(!(parallel %% 1 == 0),parallel < 1)) stop("'parallel' should be a positive integer")
   
-  #if parallel in OSX
-  if(all(parallel > 1, !Sys.info()[1] %in% c("Linux","Windows"))) {
-    parallel <- 1
-    ("creating images is not compatible with parallel computing (parallel > 1) in OSX (mac)")
-  }
-  
-  # If parallel was called
-  if(parallel > 1)
-  { options(warn = -1)
-     
-       
-        if(Sys.info()[1] == "Windows"){ 
-           
-          lapp <- pbapply::pblapply} else lapp <- function(X, FUN) parallel::mclapply(X, FUN, mc.cores = parallel)} else lapp <- pbapply::pblapply
-  
-  options(warn = 0)
-  
-  # Create spectrograms overlaid with start and end times from manualoc()
-  if(parallel == 1) ("Creating spectrograms from selections:")
-  
-  
-  invisible(lapp(1:length(sound.files), function(i){
+  #create function to run within Xapply functions downstream     
+  specreFUN <- function(X, i, mar, flim, xl, picsize, res, wl, ovlp, cexlab){
     
     # Read sound files, initialize frequency and time limits for spectrogram
-    r <- tuneR::readWave(file.path(getwd(), sound.files[i]), header = T)
+    r <- tuneR::readWave(as.character(X$sound.files[i]), header = TRUE)
     f <- r$sample.rate
-    t <- c(start[i] - mar, end[i] + mar)
+    t <- c(X$start[i] - mar, X$end[i] + mar)
     
     mar1 <- mar
-    mar2 <- mar1 + end[i] - start[i]
+    mar2 <- mar1 + X$end[i] - X$start[i]
     
     if (t[1] < 0) { 
       mar1 <- mar1  + t[1]
@@ -187,16 +162,16 @@ specreator <- function(X, wl = 512, flim = c(0, 22), wn = "hanning", pal = rever
     
     # Spectrogram width can be proportional to signal duration
     if(propwidth){
-      if(it == "tiff")  tiff(filename = paste(sound.files[i],"-", selec[i], "-", ".tiff", sep = ""), 
+      if(it == "tiff")  tiff(filename = paste(X$sound.files[i],"-", X$selec[i], "-", ".tiff", sep = ""), 
                              width = (10.16) * ((t[2]-t[1])/0.27) * xl * picsize, height = (10.16) * picsize, units = "cm", res = res) else
-                               jpeg(filename = paste(sound.files[i],"-", selec[i], "-", ".jpeg", sep = ""), 
+                               jpeg(filename = paste(X$sound.files[i],"-", X$selec[i], "-", ".jpeg", sep = ""), 
                                     width = (10.16) * ((t[2]-t[1])/0.27) * xl * picsize, height = (10.16) * picsize, 
                                     units = "cm", res = res) 
       
     } else {
-      if(it == "tiff")  tiff(filename = paste(sound.files[i],"-", selec[i], "-", ".tiff", sep = ""), 
+      if(it == "tiff")  tiff(filename = paste(X$sound.files[i],"-", X$selec[i], "-", ".tiff", sep = ""), 
                              width = (10.16) * xl * picsize, height = (10.16) * picsize, units = "cm", res = res) else
-                               jpeg(filename = paste(sound.files[i],"-", selec[i], "-", ".jpeg", sep = ""), 
+                               jpeg(filename = paste(X$sound.files[i],"-", X$selec[i], "-", ".jpeg", sep = ""), 
                                     width = (10.16) * xl * picsize, height = (10.16) * picsize, units = "cm", res = res)
     }
     
@@ -214,20 +189,60 @@ specreator <- function(X, wl = 512, flim = c(0, 22), wn = "hanning", pal = rever
     par(oma = outer.mar)
     
     # Generate spectrogram using seewave 
-    seewave::spectro(tuneR::readWave(as.character(sound.files[i]), from = t[1], to = t[2], units = "seconds") , f = f, wl = wl, ovlp = ovlp, collevels = seq(-40, 0, 0.5), heights = hts, wn = "hanning", 
+    seewave::spectro(tuneR::readWave(as.character(X$sound.files[i]), from = t[1], to = t[2], units = "seconds") , f = f, wl = wl, ovlp = ovlp, collevels = seq(-40, 0, 0.5), heights = hts, wn = "hanning", 
                      widths = wts, palette = pal, osc = osci, grid = gr, scale = sc, collab = "black", 
                      cexlab = cexlab, cex.axis = 1, flim = fl, tlab = "Time (s)", 
                      flab = "Frequency (kHz)", alab = "", trel = FALSE)
     
     # Add title to spectrogram
-    if(title) if(!is.na(selcom[i]))
-      title(paste(sound.files[i], "-", selec[i], "-", selcom[i], sep = ""), cex.main = cexlab) else
-        title(paste(sound.files[i], "-", selec[i], sep = ""), cex.main = cexlab)
+    if(title) if(!is.null(X$sel.comment[i]))
+      title(paste(X$sound.files[i], "-", X$selec[i], "-", X$sel.comment[i], sep = ""), cex.main = cexlab) else
+        title(paste(X$sound.files[i], "-", X$selec[i], sep = ""), cex.main = cexlab)
     
     # Plot lines to visualize selections (start and end of signal)
+      if(line)    
         abline(v = c(mar1, mar2), col = "red", lwd = 3, lty = "dashed")
     invisible() # execute par(old.par) 
     dev.off()
   }
-  ))
+
+  # Run parallel in windows
+  if(parallel > 1) {
+    if(Sys.info()[1] == "Windows") {
+      
+      i <- NULL #only to avoid non-declared objects
+      
+      cl <- parallel::makeCluster(parallel)
+      
+      doParallel::registerDoParallel(cl)
+      
+      sp <- foreach::foreach(i = 1:nrow(X)) %dopar% {
+        specreFUN(X = X, i = i, mar = mar, wl = wl, flim = flim, xl = xl, picsize = picsize, res = res, ovlp = ovlp, cexlab = cexlab)
+      }
+      
+      parallel::stopCluster(cl)
+      
+    } 
+    if(Sys.info()[1] == "Linux") {    # Run parallel in other operating systems
+      
+      sp <- parallel::mclapply(1:nrow(X), function (i) {
+        specreFUN(X = X, i = i, mar = mar, wl = wl, flim = flim, xl = xl, picsize = picsize, res = res, ovlp = ovlp, cexlab = cexlab)
+      })
+    }
+    if(!any(Sys.info()[1] == c("Linux", "Windows")))
+    {
+      cl <- parallel::makeForkCluster(getOption("cl.cores", parallel))
+      
+      sp <- foreach::foreach(i = 1:nrow(X)) %dopar% {
+        specreFUN(X = X, i = i, mar = mar, wl = wl, flim = flim, xl = xl, picsize = picsize, res = res, ovlp = ovlp, cexlab = cexlab)
+      }
+      
+      parallel::stopCluster(cl)
+      
+    }
+    }
+  else {
+    sp <- pbapply::pblapply(1:nrow(X), function(i) specreFUN(X = X, i = i, mar = mar, wl = wl, flim = flim, xl = xl, picsize = picsize, res = res, ovlp = ovlp, cexlab = cexlab))
+  }
+          
 }
